#include <iostream>
#include <string>
#include <vector>
#include <windows.h>
#include "xorstr.hpp" // Include our XOR string header

// ============= FUNCTIONS TO BE MONITORED BY FUNCAP =============

// FUNCAP SHOULD TRACK: str1, str2, return value
std::string concatenateStrings(const std::string& str1, const std::string& str2) {
    std::cout << "Funcap should capture arguments: " << str1 << ", " << str2 << std::endl;
    // Store string in memory to make it easier for Funcap to detect
    char* buffer = new char[str1.length() + str2.length() + 1];
    strcpy(buffer, str1.c_str());
    strcat(buffer, str2.c_str());
    std::string result = buffer;
    delete[] buffer;
    return result;
}

// FUNCAP SHOULD TRACK: source, result, memory leak
char* allocateAndCopyString(const char* source) {
    size_t len = strlen(source);
    char* result = new char[len + 1];
    memcpy(result, source, len + 1);
    // Add some delay to give Funcap time to capture
    for(volatile int i = 0; i < 10000000; i++) {}
    std::cout << "Funcap should capture source: " << source << std::endl;
    std::cout << "Funcap should capture result: " << result << std::endl;
    return result;
}

// FUNCAP SHOULD TRACK: size, ptr, malloc() call
void* customMemoryAllocation(size_t size) {
    void* ptr = malloc(size);
    // Put a marker string in memory to help Funcap identify the allocation
    if (ptr && size > 20) {
        strcpy((char*)ptr, "FUNCAP_MARKER");
    }
    std::cout << "Funcap should capture size: " << size << " bytes" << std::endl;
    std::cout << "Funcap should capture address: " << ptr << std::endl;
    return ptr;
}

// FUNCAP SHOULD TRACK: strings vector
void processStringVector(const std::vector<std::string>& strings) {
    std::cout << "Funcap should capture vector of size: " << strings.size() << std::endl;
    // Create a buffer with all strings concatenated to make it easier to detect
    std::string allStrings;
    for (const auto& str : strings) {
        allStrings += str + "|";
        std::cout << xorstr_("Processing: ") << str << std::endl;
    }
    // Keep the string in memory briefly
    char* buffer = new char[allStrings.length() + 1];
    strcpy(buffer, allStrings.c_str());
    for(volatile int i = 0; i < 10000000; i++) {}
    delete[] buffer;
}

// FUNCAP SHOULD TRACK: all arguments (a, b, c, d) and return value
int complexFunction(int a, const char* b, void* c, double d) {
    // Create a formatted string with all parameters to help Funcap
    char buffer[256];
    sprintf(buffer, "a=%d, b=%s, c=%p, d=%f", a, b, c, d);
    std::cout << xorstr_("Complex function called with: ") << buffer << std::endl;

    int result = a + static_cast<int>(strlen(b)) + static_cast<int>(d);
    std::cout << "Funcap should capture result: " << result << std::endl;
    return result;
}

// FUNCAP SHOULD TRACK: GetSystemTime call and st structure
void testWindowsAPI() {
    SYSTEMTIME st;
    std::cout << "Funcap should capture GetSystemTime call" << std::endl;
    GetSystemTime(&st);

    // Format the time into a string to make it easier to detect
    char timeBuffer[128];
    sprintf(timeBuffer, "FUNCAP_TIME: %04d-%02d-%02d %02d:%02d:%02d",
            st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
    std::cout << xorstr_("Current time: ") << timeBuffer << std::endl;
}

// FUNCAP SHOULD TRACK: recursive calls and call stack
int fibonacci(int n) {
    // Add a marker string to help Funcap track the recursion
    char marker[32];
    sprintf(marker, "FUNCAP_FIBO_%d", n);
    std::cout << "Funcap should capture recursion depth: " << marker << std::endl;
    if (n <= 1) return n;
    return fibonacci(n - 1) + fibonacci(n - 2);
}

// FUNCAP SHOULD TRACK: XOR encrypted string
void processEncryptedData(const char* encrypted_data) {
    // Copy the data to ensure it's in memory long enough
    char* buffer = new char[strlen(encrypted_data) + 20];
    strcpy(buffer, "FUNCAP_XOR:");
    strcat(buffer, encrypted_data);
    std::cout << "Funcap should capture encrypted data: " << buffer << std::endl;

    // Add delay to give Funcap time to capture
    for(volatile int i = 0; i < 10000000; i++) {}
    delete[] buffer;
}

int main() {
    std::cout << xorstr_("======= FUNCAP TESTING START =======") << std::endl;

    // Test 1: String concatenation
    std::cout << xorstr_("\n--- Test 1: String Concatenation ---") << std::endl;
    std::string result = concatenateStrings(xorstr_("Hello, "), xorstr_("Funcap!"));
    std::cout << xorstr_("Concatenation result: ") << result << std::endl;
    // FUNCAP SHOULD OUTPUT: "Hello, ", "Funcap!", "Hello, Funcap!"

    // Test 2: Memory allocation and string copying
    std::cout << xorstr_("\n--- Test 2: Memory Allocation and String Copying ---") << std::endl;
    const char* originalString = xorstr_("FUNCAP_TEST_STRING_FOR_MEMORY_ALLOCATION");
    char* copiedString = allocateAndCopyString(originalString);
    std::cout << xorstr_("Copied string: ") << copiedString << std::endl;
    // FUNCAP SHOULD OUTPUT: "FUNCAP_TEST_STRING_FOR_MEMORY_ALLOCATION", copiedString address

    // Test 3: Custom memory allocation
    std::cout << xorstr_("\n--- Test 3: Custom Memory Allocation ---") << std::endl;
    void* memory = customMemoryAllocation(128);
    if (memory) {
        memset((char*)memory + 13, 'A', 10);  // Fill bytes after marker with 'A'
        ((char*)memory)[23] = '\0';  // Null-terminate
        std::cout << xorstr_("Memory content: ") << (char*)memory << std::endl;
        free(memory);
        // FUNCAP SHOULD OUTPUT: malloc(128) call, memory address
    }

    // Test 4: String vector processing
    std::cout << xorstr_("\n--- Test 4: String Vector Processing ---") << std::endl;
    std::vector<std::string> stringVector = {
        xorstr_("FUNCAP_STRING_1"),
        xorstr_("FUNCAP_STRING_2"),
        xorstr_("FUNCAP_STRING_3_!@#$%^&*()"),
        xorstr_("FUNCAP_STRING_4_VERY_LONG_TEXT_TO_TEST_DETECTION")
    };
    processStringVector(stringVector);
    // FUNCAP SHOULD OUTPUT: vector content, all 4 strings

    // Test 5: Complex function
    std::cout << xorstr_("\n--- Test 5: Complex Function ---") << std::endl;
    int complexResult = complexFunction(42, xorstr_("FUNCAP_COMPLEX_ARGUMENT"), memory, 3.14159);
    std::cout << xorstr_("Complex function result: ") << complexResult << std::endl;
    // FUNCAP SHOULD OUTPUT: 42, "FUNCAP_COMPLEX_ARGUMENT", memory, 3.14159, result

    // Test 6: Windows API
    std::cout << xorstr_("\n--- Test 6: Windows API ---") << std::endl;
    testWindowsAPI();
    // FUNCAP SHOULD OUTPUT: GetSystemTime call, SYSTEMTIME structure data

    // Test 7: Recursive function
    std::cout << xorstr_("\n--- Test 7: Recursive Function ---") << std::endl;
    int fibResult = fibonacci(7); // Reduced to 7 for fewer calls
    std::cout << xorstr_("Fibonacci(7): ") << fibResult << std::endl;
    // FUNCAP SHOULD OUTPUT: fibonacci call stack with different n values

    // Test 8: Encrypted data processing
    std::cout << xorstr_("\n--- Test 8: Encrypted Data Processing ---") << std::endl;
    const char* encrypted = xorstr_("FUNCAP_THIS_IS_SECRET_DATA_XOR_ENCRYPTED");
    processEncryptedData(encrypted);
    // FUNCAP SHOULD OUTPUT: "FUNCAP_THIS_IS_SECRET_DATA_XOR_ENCRYPTED" in decrypted form

    // Cleanup
    delete[] copiedString;

    std::cout << xorstr_("\n======= FUNCAP TESTING COMPLETE =======") << std::endl;
    std::cout << xorstr_("Press Enter to exit...");
    std::cin.get();

    return 0;
}

/*
Expected Funcap output examples:

1. Function Call Log:
[CALL] concatenateStrings(str1="Hello, ", str2="Funcap!")
[RETURN] concatenateStrings → "Hello, Funcap!"

2. Memory Allocation:
[CALL] allocateAndCopyString(source="FUNCAP_TEST_STRING_FOR_MEMORY_ALLOCATION")
[ALLOC] 0x00A3B4C0, size=32, by=allocateAndCopyString
[RETURN] allocateAndCopyString → 0x00A3B4C0 "FUNCAP_TEST_STRING_FOR_MEMORY_ALLOCATION"

3. Malloc Tracking:
[CALL] customMemoryAllocation(size=128)
[ALLOC] malloc(128) → 0x00A3B5D0
[RETURN] customMemoryAllocation → 0x00A3B5D0

4. Vector Processing:
[CALL] processStringVector(strings=std::vector<std::string>{
    "FUNCAP_STRING_1",
    "FUNCAP_STRING_2",
    "FUNCAP_STRING_3_!@#$%^&*()",
    "FUNCAP_STRING_4_VERY_LONG_TEXT_TO_TEST_DETECTION"
})
[RETURN] processStringVector → void

5. Complex Function:
[CALL] complexFunction(a=42, b="FUNCAP_COMPLEX_ARGUMENT", c=0x00000000, d=3.14159)
[RETURN] complexFunction → 61

6. External API Calls:
[CALL] GetSystemTime(lpSystemTime=0x0012FE40)
[DATA] SYSTEMTIME at 0x0012FE40 = {wYear=2023, wMonth=5, wDay=10, wHour=15, wMinute=30, wSecond=45, ...}

7. Recursive Function:
[CALL] fibonacci(n=7)
  [CALL] fibonacci(n=6)
    [CALL] fibonacci(n=5)
      // ... additional nested calls ...
    [RETURN] fibonacci → 5
  [RETURN] fibonacci → 8
[RETURN] fibonacci → 13

8. Encrypted String Detection:
[CALL] processEncryptedData(encrypted_data="FUNCAP_THIS_IS_SECRET_DATA_XOR_ENCRYPTED")
[DATA] encrypted_data = "FUNCAP_THIS_IS_SECRET_DATA_XOR_ENCRYPTED" (decrypted from XOR-encrypted string)
[RETURN] processEncryptedData → void

9. Memory Leaks and Cleanup:
[FREE] 0x00A3B4C0, allocated by=allocateAndCopyString
[FREE] 0x00A3B5D0, allocated by=malloc

Note: The actual addresses will vary in each execution, and Funcap should display
the decrypted content of strings even though they are XOR-encrypted in the code.
*/
